% parametri, SI jedinice osim energije (eV)
e0=0; tx=-1; ty=-2;
NL=20; NW=[1,2,5,10]; % duljina i sirine nanovrpce
zplus=1i*1e-6;
E=linspace(-7,7,1001);
errmax=1e-6;

TRM=zeros(length(NW),length(E));
DOS=zeros(length(NW),length(E));
DD=zeros(1,length(E));
TT=zeros(1,length(E));

for m=1:length(NW)
    
    % matrice TB modela: super-celija, interakcije i ukupni hamiltonijan
    alpha = e0*diag(ones(1,NW(m))) + ty*diag(ones(1,NW(m)-1),1)...
        + ty*diag(ones(1,NW(m)-1),-1);
    beta = tx*diag(ones(1,NW(m)));
    H = kron(diag(ones(1,NL)),alpha) + kron(diag(ones(1,NL-1),1),beta)...
        + kron(diag(ones(1,NL-1),-1),beta');

    for k=1:length(E)
        
        % racunanje povrsinske GF za lijevi kontakt, Sancho-Rubio metoda
        A=beta; B=beta'; C=alpha; D=alpha;
        err = 100;
        while(err>errmax)
            X = inv((E(k)+zplus)*eye(NW(m))-C);
            Anew = A * X * A; Bnew = B * X * B;
            Cnew = C + A*X*B + B*X*A;
            Dnew = D + B*X*A;
            err1 = max(max(abs(A-Anew))); err2 = max(max(abs(B-Bnew)));
            err3 = max(max(abs(C-Cnew))); err4 = max(max(abs(D-Dnew)));
            err=max([err1,err2,err3,err4]);
            A=Anew; B=Bnew; C=Cnew; D=Dnew;
        end
        gs1=inv((E(k)+zplus)*eye(NW(m))-Dnew);
        sigma1 = beta'*gs1*beta; % lijevi kontakt
        clear A B C D X Anew Bnew Cnew Dnew
        
        % racunanje povrsinske GF za desni kontakt, Sancho-Rubio metoda
        A=beta; B=beta'; C=alpha; D=alpha;
        err = 100; %iter=1;
        while(err>errmax)
            %iter
            X = inv((E(k)+zplus)*eye(NW(m))-C);
            Anew = A * X * A; Bnew = B * X * B;
            Cnew = C + A*X*B + B*X*A;
            Dnew = D + A*X*B;
            err1 = max(max(abs(A-Anew))); err2 = max(max(abs(B-Bnew)));
            err3 = max(max(abs(C-Cnew))); err4 = max(max(abs(D-Dnew)));
            err=max([err1,err2,err3,err4]);
            A=Anew; B=Bnew; C=Cnew; D=Dnew;
        end
        gs2=inv((E(k)+zplus)*eye(NW(m))-Dnew);
        sigma2 = beta*gs2*beta'; % desni kontakt
        clear A B C D X Anew Bnew Cnew Dnew
        
        % matrice vl. en. kontakata
        sigL = kron(diag([1 zeros(1,NL-1)]),sigma1);
        sigR = kron(diag([zeros(1,NL-1) 1]),sigma2);
        
        % matrice prosirenja
        gamL=1i*(sigL-sigL');
        gamR=1i*(sigR-sigR');
        
        % Greenova i spektralna funkcija        
        G=inv(((E(k)+zplus)*eye(NW(m)*NL))-H-sigL-sigR);
        AA=1i*(G-G');

        % gustoca stanja i transmisija
        DD(k)=2*real(trace(AA))/(2*pi); % u brojniku 2 za spin
        TT(k)=real(trace(gamL*G*gamR*G'));
            
    end
    
    DOS(m,:)=DD;
    TRM(m,:)=TT;
    
end

figure(1),...
    plot(E,TRM(1,:),'b-',E,TRM(2,:),'r--',E,TRM(3,:),'g-.',E,TRM(4,:),...
    'k:','LineWidth',2), grid on, xlim([-8 8]), ylim([0 6]), xlabel('E (eV)'),...
    ylabel('T(E)'), legend('N_{W} = 1','N_{W} = 2','N_{W} = 5','N_{W} = 10',...
    'Location','northeast'), title('i\eta = i*10^{-6}, N_{L} = 20');

figure(2),...
    plot(E,DOS(1,:),'b-',E,DOS(2,:),'r--',E,DOS(3,:),'g-.',E,DOS(4,:),...
    'k:','LineWidth',2), grid on, xlim([-8 8]), ylim([0 200]), xlabel('E (eV)'),...
    ylabel('DOS(E) (eV^{-1})'), legend('N_{W} = 1','N_{W} = 2','N_{W} = 5',...
    'N_{W} = 10','Location','northeast'), title('i\eta = i*10^{-6}, N_{L} = 20');







