% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
m=0.25*9.1e-31;
IE=(q*q)/(2*pi*hbar); % strujna konstanta, 2 za spin NIJE ukljuceno
Ef=0.1;
kT=0.025;
UB0=0.4;
zplus=1i*1e-12;

% definicija strukture
a=0.1e-9; 
LE=10e-9; LC=10e-9; LB=1e-9; LQW=3e-9;
Ttotal=LE+LC+LB+LQW;
NE=floor(LE/a); NC=floor(LC/a); NB=floor(LB/a); NQW=floor(LQW/a);
Np=NE+NB+NQW+NB+NC;
XX=a*1e9*[1:1:Np];

% parametar interakcije za EMA hamiltonijan
t0=(hbar^2)/(2*m*(a^2)*q);

% konstrukcija EMA hamiltonijana
% potencijalni dio, barijere i jama
UB=[zeros(NE,1); UB0*ones(NB,1); zeros(NQW,1); UB0*ones(NB,1); zeros(NC,1)];
% kineticki dio
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));
% ukupni H
H = T + diag(UB);

% definicija energijskog spektra
NEE=401;
E=linspace(-0.5,0.8,NEE);
dE=E(2)-E(1);

% naponske tocke za racunanje struje
NV=41; VV=linspace(0,0.4,NV); II=zeros(1,NV);

for iV=1:NV

    V=VV(iV); mu1=Ef; mu2=Ef-V;
    % profil potencijala od narinutog napona
    U1=V*[0*ones(1,NE) linspace(0,-1,(NB+NQW+NB)) -1*ones(1,NC)];
    U1=U1';

    % Fermijeve funkcije za EMA slucaj, 2 za spin ukljuceno, 1/m2
    f1=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu1-E)./kT)); 
    f2=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu2-E)./kT));

    I=0;
    for k=1:NEE
        
        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka);
        gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka);
        gam2=1i*(sig2-sig2');

        G=inv(((E(k)+zplus)*eye(Np))-H-diag(U1)-sig1-sig2);
        TRM(k)=real(trace(gam1*G*gam2*G'));

        I=I+(dE*IE*TRM(k)*(f1(k)-f2(k))); % struja, A/m2

    end

    II(iV)=I;
     
end

figure(1),...
    plot(VV,II,'b','LineWidth',2), axis([0 0.5 0 max(II)]), xlabel('U (V)'),...
    ylabel('I (A/m^{2})'), grid on;




