% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
m=0.25*9.1e-31;
IE=(q*q)/(2*pi*hbar); % strujna konstanta, 2 za spin NIJE ukljuceno
Ef=0.1;
kT=0.025;
UB0=0.4;
zplus=1i*1e-12;

% definicija strukture
a=0.1e-9; 
LE=10e-9; LC=10e-9; LB=1e-9; LQW=3e-9;
Ttotal=LE+LC+LB+LQW;
NE=floor(LE/a); NC=floor(LC/a); NB=floor(LB/a); NQW=floor(LQW/a);
Np=NE+NB+NQW+NB+NC;
XX=a*1e9*[1:1:Np];

% parametar interakcije za EMA hamiltonijan
t0=(hbar^2)/(2*m*(a^2)*q);

% konstrukcija EMA hamiltonijana
% potencijalni dio, barijere i jama
UB=[zeros(NE,1); UB0*ones(NB,1); zeros(NQW,1); UB0*ones(NB,1); zeros(NC,1)];
% kineticki dio
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));
% ukupni H
H = T + diag(UB);

% definicija energijskog spektra
NEE=401;
E=linspace(-0.5,0.8,NEE);
dE=E(2)-E(1);

% tri naponske tocke za crtanje gustoca stanja, elektrona i struje
VV=[0.11,0.20,0.38];
NV=length(VV);

for iV=1:NV

    V=VV(iV); mu1=Ef; mu2=Ef-V;
    % profil potencijala od narinutog napona
    U1=V*[0*ones(1,NE) linspace(0,-1,(NB+NQW+NB)) -1*ones(1,NC)];
    U1=U1';

    % samo za plotanje uz transmisiju
    fo1=1./(1+exp((E-mu1)./kT));
    fo2=1./(1+exp((E-mu2)./kT));
		
    % Fermijeve funkcije za EMA slucaj, 2 za spin ukljuceno, 1/m2
    f1=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu1-E)./kT)); 
    f2=(2*m*kT*q/(2*pi*hbar^2)).*log(1+exp((mu2-E)./kT));

    I=0;
    for k=1:NEE
        
        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka);
        gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka);
        gam2=1i*(sig2-sig2');

        G=inv(((E(k)+zplus)*eye(Np))-H-diag(U1)-sig1-sig2);
        TRM(k)=real(trace(gam1*G*gam2*G'));
        
        % struja, A/m2
        I=I+(dE*IE*TRM(k)*(f1(k)-f2(k))); 
        
        % gustoca stanja, 2 za spin, normalizirano s a, 1/eV-m 
        DD=diag(1i*(G-G'))/(2*pi); 
        LDOS(k,1:Np)=2*(real(DD))'/a; 
        
        % gustoca struje, spin ukljucen u f1/f2, A/eV-m2
        II0 = diag(gam1*1i*(G-G')*f1(k)-gam1*G*(gam1*f1(k)+gam2*f2(k))*G');
        IDOS(k,1:Np)=IE*(real(II0))'; 
        
        % gustoca nosilaca, spin ukljucen u f1/f2, normalizirano, 1/eV-m3
        N0=diag((f1(k)*G*gam1*G'+f2(k)*G*gam2*G'))/(2*pi); 
        NDOS(k,1:Np)=(real(N0))'/a; 
  
    end
        
        IIDOS=IDOS(:,1)*ones(1,Np); % energijska raspodjela struje je svuda
                                    % ista zbog koherentnog transporta
        
        figure(iV), ...
            set(gcf, 'Units', 'Centimeters', 'Position', [0, 0, 40, 12], ...
            'PaperUnits', 'Centimeters', 'PaperSize', [40, 12])
            subplot(1,4,3), plot(TRM,E,'b-',(fo1-fo2),E,'r:','LineWidth',2),...
                axis([0 1.2 -0.5 0.8]), xlabel('T(E)'), ylabel('E (eV)'), ...
                title(['U = ',num2str(V),' V']), legend('T(E)','f_{L}(E) - f_{R}(E)',...
                'Location','southeast'),...
            subplot(1,4,1), pcolor(XX,E,LDOS), colorbar('Location',...
                'EastOutside'), colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
                title('LDOS (1/(eV*m))'), xlabel('x (nm)'), ylabel('E (eV)'),...
                shading interp, hold on, plot(XX,U1+UB,'w--','LineWidth',2),...
            subplot(1,4,2), pcolor(XX,E,NDOS), colorbar('Location',...
                'EastOutside'), colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
                title('NDOS (1/(eV*m^{3}))'), xlabel('x (nm)'), ylabel('E (eV)'),...
                shading interp, hold on, plot(XX,U1+UB,'w--','LineWidth',2),...
            subplot(1,4,4), pcolor(XX,E,IIDOS), colorbar('Location',...
                'EastOutside'), colormap jet, axis([0 Ttotal/1e-9 -0.5 0.8]),...
                title('IDOS (A/(eV*m^{2}))'), xlabel('x (nm)'), ylabel('E (eV)'),...
                shading interp, hold on, plot(XX,U1+UB,'w--','LineWidth',2);
        
end





