% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
kT=0.025;
m=0.041*9.1e-31; % 0.041 ef. masa za InGaAs
Ef=0.1;
UB=0.5; % visina barijere
zplus=1i*1e-12;

% definicija strukture
a=0.5e-9;
LE=20e-9; LC=20e-9; LB=12e-9; LW=12e-9;
Ttotal=4*LB+3*LW+LE+LC;
NEm=floor(LE/a); NC=floor(LC/a); NB=floor(LB/a); NW=floor(LW/a);
Np=4*NB+3*NW+NEm+NC;
XX=a*1e9*[1:1:Np]-a*1e9;

% parametar interakcije za EMA hamiltonijan
t0=(hbar^2)/(2*m*(a^2)*q);

% konstrukcija EMA hamiltonijana
UB=[zeros(1,NEm) kron([1 0 1 0 1 0 1], UB*ones(1,NB)) zeros(1,NC)]';
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));
H=T+diag(UB);

% definicija energijskog spektra
NE=1001; E=linspace(-0.7,1.2,NE);

% predalokacija za ubrzanje racunanja
LDOS=zeros(length(E),Np);

% jedna naponska tocka za crtanje lokalne gustoce stanja
% napon za prag laserskog efekta
VV=1.1; NV=length(VV);

for iV=1:NV

    V=VV(iV);
    % profil potencijala od narinutog napona
    U1=V*[0.5*ones(1,NEm) linspace(0.5,-0.5,(4*NB+3*NW)) -0.5*ones(1,NC)];
    U1=U1';
		
    for k=1:NE

        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka); gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka); gam2=1i*(sig2-sig2');

        G=inv(((E(k)+zplus)*eye(Np))-H-diag(U1)-sig1-sig2); % 1/eV

        DD=diag(1i*(G-G'))/(2*pi); 
        LDOS(k,1:Np)=2*(real(DD))'/a; % LDOS, 2 za spin, normalizirano, 1/eV-m

    end

    figure(1),...
        pcolor(XX,E,log10(LDOS)), colorbar('Location','EastOutside'),...
        colormap jet, axis([0 Ttotal/1e-9 -0.7 1.2]),...
        title('log10(LDOS) (eV^{-1}m^{-1})'),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        plot(XX,U1+UB,'w--','LineWidth',2);%,...
               
end




