% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
m=0.25*9.1e-31;
Ef=0.15;
kT=0.025;
zplus=1i*1e-12;
D=0.1; % jakost rasprsenja
Eop=0.020; % energija optickog fonona

% definicija strukture i hamiltonijana
a=3e-10;
Np=42;
UB=zeros(Np,1); % profil barijera za primjer 5.8 i 5.9
t0=(hbar^2)/(2*m*(a^2)*q);
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));
XX=a*1e9*[0:1:Np-1];
Ttotal=(Np-2)*a;

% napon, kvazi-Fermijeve razine i profil potencijala
V=0.1;
mu1=Ef+(V/2); mu2=Ef-(V/2);
U1=V*[0.5*ones(1,1) linspace(0.5,-0.5,Np-2) -0.5*ones(1,1)]';

% definicija energijskog spektra
NE=101;
E=linspace(-0.1,0.4,NE);
dE=E(2)-E(1);
f1=1./(1+exp((E-mu1)./kT));
f2=1./(1+exp((E-mu2)./kT));


% predalokacija radi brzeg racunanja
error=1;
Gn=zeros(Np,NE); Gp=zeros(Np,NE);
IIDOS=zeros(NE,Np); Gnx=zeros(Np,Np,NE);
sigin1=zeros(Np,NE); sigout1=zeros(Np,NE);
sigin2=zeros(Np,NE); sigout2=zeros(Np,NE);

% pocetno rjesenje za Sigma_in_S i Sigma_out_S
siginS=0*ones(Np,NE);
sigoutS=0*ones(Np,NE);

% samokonzistentna petlja za neelasticna rasprsenja
while error>1e-3
    for k=1:NE
        
        sig1=zeros(Np); sig2=zeros(Np);
        
        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka); gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka); gam2=1i*(sig2-sig2');

        sigin1(:,k)=f1(k)*diag(gam1);
        sigin2(:,k)=f2(k)*diag(gam2);
        sigout1(:,k)=(1-f1(k))*diag(gam1);
        sigout2(:,k)=(1-f2(k))*diag(gam2);
        
        gamS=siginS(:,k)+sigoutS(:,k);

        G=inv((E(k)+zplus)*eye(Np)-T-diag(U1+UB)-sig1-sig2+1i*0.5*diag(gamS));
        A=1i*(G-G');
        Gn(:,k)=real(diag(G*diag(sigin1(:,k)+sigin2(:,k)+siginS(:,k))*G'));
        Gp(:,k)=diag(A)-Gn(:,k);

        % cjelovita 3D matrica elektronske korelacijske funkcije
        % za plotanje gustoce struje J(r,E)
        Gnx(:,:,k)=(G*((f1(k)*gam1)+(f2(k)*gam2)+diag(siginS(:,k)))*G');
        
    end
    
    % energijski pomak za emisiju i apsorpciju
    off=floor(Eop/dE)+1;
    % broj fonona
    Nph=1/(exp(dE*off/kT)-1);
    
    % generiranje Sigma_in_S i Sigma_out_S s ukljucenim doprinosima
    % emisije i apsorpcije za elektronsku i upljinsku korelacijsku
    % funkciju
    Gnemi=Gn(:,[off+1:NE]);  Gnemi=[Gnemi zeros(Np,off)];
    Gnabs=Gn(:,[1:NE-off]);  Gnabs=[zeros(Np,off) Gnabs];
    Gpabs=Gp(:,[off+1:NE]);  Gpabs=[Gpabs zeros(Np,off)];
    Gpemi=Gp(:,[1:NE-off]);  Gpemi=[zeros(Np,off) Gpemi];

    siginSnova=((Nph+1)*D*Gnemi)+(Nph*D*Gnabs);
    sigoutSnova=((Nph+1)*D*Gpemi)+(Nph*D*Gpabs);
    
    err1=sum(sum(abs(siginSnova-siginS)));
    err2=sum(sum(abs(sigoutSnova-sigoutS)));
    error=max([err1,err2])
    
    siginS=siginSnova;
    sigoutS=sigoutSnova;
    
end

% racunanje gustoce struje J(r,E) iz korelacijske funkcije Gn
for k=1:NE
    for m=1:Np-1
        IIDOS(k,m)=real(1i*(t0*Gnx(m,m+1,k)-t0*Gnx(m+1,m,k)));
    end
end
Idosmax=max(max(IIDOS)); IIDOS=IIDOS./Idosmax;

I1=real((sigout1.*Gn)-(sigin1.*Gp)); I1=sum(I1);
I2=real((sigout2.*Gn)-(sigin2.*Gp)); I2=sum(I2);
Imax=max(max(abs(I1)),max(abs(I2)));


figure(1),...
    plot(I1./Imax,E,'b',I2./Imax,E,'r--','LineWidth',2),...
    xlabel('Normalizirana gustoca struje'),...
    ylabel('E (eV)'), grid on, axis([-1.2 1.2 -0.1 0.4]),...
    text(-0.9,0.3,'lijevi kontakt'), text(0.3,0.3,'desni kontakt');

figure(2),...
    pcolor(XX,E,(IIDOS)), colorbar('Location','EastOutside'), colormap jet,...
    axis([0 (Ttotal/1e-9) -0.1 0.4]),...
    xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
    title('Gustoca struje-disipativni transport'),plot(XX,U1,'w--','LineWidth',2);
