% parametri, SI jedinice osim energije (eV)
hbar=1.06e-34;
q=1.6e-19;
m=0.25*9.1e-31;
Ef=0.1;
kT=0.025;
D=0.01; % jakost rasprsenja
Eop=0.050; % energija fonona

% definicija strukture
a=0.2e-9;
UB0=0.4;
LE=10e-9; LC=10e-9; LB=1e-9; LQW=3e-9;
Ttotal=LE+LC+2*LB+LQW;
NE=floor(LE/a); NC=floor(LC/a); NB=floor(LB/a); NQW=floor(LQW/a);
Np=NE+NB+NQW+NB+NC;
XX=a*1e9*[0:1:Np-1];

% parametar interakcije za EMA hamiltonijan
t0=(hbar^2)/(2*m*(a^2)*q);

% konstrukcija EMA hamiltonijana
UB=[zeros(1,NE) UB0*ones(1,NB) zeros(1,NQW) UB0*ones(1,NB) zeros(1,NC)];
T=(2*t0*diag(ones(1,Np)))-(t0*diag(ones(1,Np-1),1))-(t0*diag(ones(1,Np-1),-1));

% naponska tocka za racunanje struje i plotanje gustoca stanja i struje
% i potencijalni profil od narinutog napona
V=0.22;
mu1=Ef; mu2=Ef-V;
U1=V*[0*ones(1,NE) linspace(0,-1,(NB+NQW+NB)) -1*ones(1,NC)];

% definicija energijskog spektra
NEE=401;
E=linspace(-0.4,0.6,NEE);
zplus=1i*1e-6;
dE=E(2)-E(1);
% Fermijeva funkcija f_0 radi jednostavnosti
f1=1./(1+exp((E-mu1)./kT)); 
f2=1./(1+exp((E-mu2)./kT));

% predalokacija radi brzeg racunanja i inicijalizacija
siginS=zeros(Np,NEE); sigoutS=zeros(Np,NEE);
Gn=zeros(Np,NEE); Gp=zeros(Np,NEE);
IIDOS=zeros(NEE,Np); Gnx=zeros(Np,Np,NEE); LDOS=zeros(NEE,Np);
sigin1=zeros(Np,NE); sigout1=zeros(Np,NE);
sigin2=zeros(Np,NE); sigout2=zeros(Np,NE);

error=1;

while error>1e-4
    
    for k=1:NEE
        
        sig1=zeros(Np); sig2=zeros(Np);

        ck=1-((E(k)+zplus-U1(1)-UB(1))/(2*t0)); ka=acos(ck);
        sig1(1,1)=-t0*exp(1i*ka); gam1=1i*(sig1-sig1');

        ck=1-((E(k)+zplus-U1(Np)-UB(Np))/(2*t0)); ka=acos(ck);
        sig2(Np,Np)=-t0*exp(1i*ka); gam2=1i*(sig2-sig2');

        sigin1(:,k)=f1(k)*diag(gam1);
        sigin2(:,k)=f2(k)*diag(gam2);
        sigout1(:,k)=(1-f1(k))*diag(gam1);
        sigout2(:,k)=(1-f2(k))*diag(gam2);

        gamS=siginS(:,k)+sigoutS(:,k);

        G=inv(((E(k)+zplus)*eye(Np))-T-diag(U1+UB)-sig1-sig2+1i*0.5*diag(gamS));

        A=(1i*(G-G')); DD=diag(A)/(2*pi); 
        LDOS(k,1:Np)=2*(real(DD))'/a; % 2 za spin, normalizirano, 1/eV-m

        % za samokonzistentni postupak koristimo samo dijagonale od Gn
        % i Gp radi ubrzanja proracuna iteracijske petlje
        Gn(:,k)=real(diag(G*((f1(k)*gam1)+(f2(k)*gam2)+diag(siginS(:,k)))*G'));
        Gp(:,k)=diag(A)-Gn(:,k);

        % cjelovita 3D matrica elektronske korelacijske funkcije Gn(r,r',E)
        % za plotanje gustoce struje J(r,E)
        Gnx(:,:,k)=((G*((f1(k)*gam1)+(f2(k)*gam2)+diag(siginS(:,k)))*G'));

    end
    % energijski pomak za emisiju i apsorpciju
    off=floor(Eop/dE)+1;
    % broj fonona
    Nph=1/(exp(dE*off/kT)-1);
    
    % generiranje Sigma_in_S i Sigma_out_S s ukljucenim doprinosima
    % emisije i apsorpcije za elektronsku i upljinsku korelacijsku
    % funkciju
    Gnemi=Gn(:,[off+1:NEE]); Gnemi=[Gnemi zeros(Np,off)];
    Gnabs=Gn(:,[1:NEE-off]); Gnabs=[zeros(Np,off) Gnabs];
    Gpabs=Gp(:,[off+1:NEE]); Gpabs=[Gpabs zeros(Np,off)];
    Gpemi=Gp(:,[1:NEE-off]); Gpemi=[zeros(Np,off) Gpemi];

    siginSnova=(Nph+1)*D*Gnemi+Nph*D*Gnabs;
    sigoutSnova=(Nph+1)*D*Gpemi+Nph*D*Gpabs;
    
    err1=sum(sum(abs(siginSnova-siginS)))/sum(sum(abs(siginS)));
    err2=sum(sum(abs(sigoutSnova-sigoutS)))/sum(sum(abs(sigoutS)));
        
    error=max([err1,err2])
    
    siginS=siginSnova;
    sigoutS=sigoutSnova;

end


% racunanje gustoce struje J(r,E) iz korelacijske funkcije Gn(r,r',E)
for k=1:NEE
    for m=1:Np-1
        IIDOS(k,m) = q*q/(2*pi*hbar) * ...
            real(1i*(t0*Gnx(m,m+1,k) - t0*Gnx(m+1,m,k)));
        % ako se izostavi jedan "q" u brojniku koeficijenta, gustoca ce
        % biti u jedinicama A/J
    end
end

% parametri za normalizaciju struje za treci graf
I1=-1*IIDOS(:,1); I2=IIDOS(:,Np-1); Imax=max(max(abs(I1)),max(abs(I2)));


figure(1),...
    subplot(1,5,[1,2]),...
        pcolor(XX,E,LDOS), colorbar('Location','EastOutside'), colormap jet,...
        axis([0 (Ttotal/1e-9) -0.4 0.6]), title('Gustoca stanja (1/(eV*m))'),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        plot(XX,U1+UB,'w--','LineWidth',2)
    subplot(1,5,[3,4]),...
        pcolor(XX,E,IIDOS), colorbar('Location','EastOutside'), colormap jet,...
        axis([0 (Ttotal/1e-9) -0.4 0.6]), title('Gustoca struje (A/eV)'),...
        xlabel('x (nm)'), ylabel('E (eV)'), shading interp, hold on,...
        plot(XX,U1+UB,'w--','LineWidth',2)
    subplot(1,5,5),...
        plot(I1./Imax,E,'b',I2./Imax,E,'r--','LineWidth',2),...
        xlabel('Norm. gustoca struje'), ylabel('E (eV)'), grid on,...
        axis([-1.2 1.2 -0.4 0.6]), title('U = 0,22 V');

